# stdlib
from os import path
from pathlib import Path
from typing import List, Dict, Union
from sys import argv, exit, getsizeof
from subprocess import call, check_output
from json import dumps
# 3rd party
try:
  from yaml import safe_load, YAMLError
  from jira import Issue
except ModuleNotFoundError as e:
  print("Error: Missing one or more 3rd-party packages (pip install).")
  exit(1)


#───Utils────────────────────
def print_issue(issue:Issue, domain:str, new:bool, epic:bool, raw:bool=False) -> None:
  '''prints essential issue info'''
  if raw:
    print(dumps(issue.raw, sort_keys=True, indent=2))
  else:
    borderTailLength = 20
    if new:
      print(f"\n───NEW {issue.fields.issuetype.name.upper()}{'─'*(borderTailLength - len(issue.fields.issuetype.name))}")
    else:
      print(f"\n───EXISTING {issue.fields.issuetype.name.upper()}{'─'*(borderTailLength - len(issue.fields.issuetype.name) - 5)}")
    print(f"Key      │ {issue.key}")
    print(f"Url      │ {domain}/browse/{issue.key}")
    print(f"Type     │ {issue.fields.issuetype.name}")
    print(f"Created  │ {issue.fields.created}")
    print(f"Status   │ {issue.fields.status}")
    print(f"Priority │ {issue.fields.priority}")
    print(f"\n───GIT COMMANDS{'─'*(borderTailLength - 8)}")
    print(f"git checkout -b [feature|bugfix]/{issue.key}_<name-of-feature>")
    print(f"git push origin [feature|bugfix]/{issue.key}_<name-of-feature> -o merge_request.create -o merge_request.target=<parent-branch>")
    if epic:
      print(f"Ticket created under parent: {issue.fields.parent.key}")
    print(f"\nOVERVIEW:\n{issue.fields.reporter} --> {issue.fields.assignee}")
    print(f"───Summary\n{issue.fields.summary}")
    print(f"───Description\n{issue.fields.description}\n")
  return



# stdlib
from os import path
from pathlib import Path
from typing import List, Dict, Union
from sys import argv, exit, getsizeof
from subprocess import call, check_output
from pprint import pprint
# custom
from toolchain.utils import *
# 3rd party
try:
  from yaml import safe_load, YAMLError
  from jira import JIRA
except ModuleNotFoundError as e:
  print("Error: Missing one or more 3rd-party packages (pip install).")
  exit(1)


#───Commands─────────────────
def make_logic(
    config_path:str, agent_path:str, summary_path:str, description_path:str
  ) -> None:
  '''read external files, assemble jira ticket, publishes and prints summary'''

  with open(summary_path, 'r') as summary, open(description_path, 'r') as description, open(config_path, 'r') as raw_config, open(agent_path, 'r') as raw_agent:
    ## load config
    try:
      summary     = summary.read()
      description = description.read()
      config      = safe_load(raw_config)
      project     = config['project']
      priority    = config['priority']
      epicKey     = config['epicKey']
      issueType   = config['issueType']
      reporter    = config['reporter']
      assignee    = config['assignee']
      agent       = safe_load(raw_agent)
      domain      = agent['domain']
      userName    = agent['userName']
      token       = agent['token']
      if (epicKey) and (issueType.lower() == 'epic'):
        print(f"Incompatible config values - can't attach epic to epic.")
        exit(1)
    except Exception as e:
      print(f"Config error.\n{e}")
      exit(1)
    ## authenticate
    try:
      jira = JIRA(server=domain, basic_auth=(userName, token))
    except Exception as e:
      print(f'Authentication error.\n{e}')
      exit(1)
    ## build
    task_issue_blueprint = {
      'project'    : {'key': project},
      'summary'    : summary,
      'description': description,
      'issuetype'  : {'name': issueType},
      'priority'   : {'name': priority},
      'reporter'   : {'accountId': reporter},
      'assignee'   : {'accountId': assignee},
    }
    epic_issue_blueprint = {
      'project'    : {'key': project},
      'summary'    : summary,
      'description': description,
      'issuetype'  : {'name': issueType},
      'reporter'   : {'accountId': reporter},
      'assignee'   : {'accountId': assignee},
    }
    ## publish
    try:
      if issueType.lower() == 'task':
        if epicKey:
          task_issue_blueprint['parent'] = {'key': epicKey}
        newIssue = jira.create_issue(fields=task_issue_blueprint)
        print_issue(issue=newIssue, domain=domain, new=True, epic=False)
        exit(0)
      elif issueType.lower() == 'epic':
        newIssue = jira.create_issue(fields=epic_issue_blueprint)
        print_issue(issue=newIssue, domain=domain, new=True, epic=True)
        exit(0)
      else:
        print(f'Error unsupported issue type.')
        exit(1)
    except Exception as e:
      print(f'Ticket publishing error.\n{e}')

  return


def fetch_logic(
    fetch_key:str, agent_path:str, raw:bool=False
  ) -> str:
  '''takes ticket key to search for, prints summary if found'''
  with open(agent_path, 'r') as raw_agent:
    ## load config
    try:
      agent    = safe_load(raw_agent)
      domain   = agent['domain']
      userName = agent['userName']
      token    = agent['token']
    except Exception as e:
      print(f"Config error.\n{e}")
      exit(1)
    ## authenticate
    try:
      jira = JIRA(server=domain, basic_auth=(userName, token))
    except Exception as e:
      print(f"Authentication error.\n{e}")
      exit(1)
    ## fetch
    issue = jira.issue(fetch_key)
    print_issue(issue=issue, domain=domain, new=False, epic=False, raw=raw)
  return

def prune_watched_logic(
  agent_path:str
) -> None:
  '''stops watching all watched issues of status DONE, prints list of pruned issues'''
  with open(agent_path, 'r') as raw_agent:
    ## load config
    try:
      agent    = safe_load(raw_agent)
      domain   = agent['domain']
      userName = agent['userName']
      token    = agent['token']
    except Exception as e:
      print(f"Config error.\n{e}")
      exit(1)
    ## authenticate
    try:
      jira = JIRA(server=domain, basic_auth=(userName, token))
    except Exception as e:
      print(f"Authentication error.\n{e}")
      exit(1)
    ## prune
    watched_issues = jira.search_issues('watcher = currentUser() AND resolution = Done')
    for issue in watched_issues:
      issue.update(fields={'watches.isWatching': 'false'})
      print(f'- stopped watching {issue.fields.summary}')
  return



#!/usr/bin/env python

'''README
Usage:
  # pip install powerjira
  # pip install --upgrade powerjira

  from powerjira import powerjira
  from sys import argv, exit

  config_path      = 'config.yml'
  agent_path       = 'agent.yml'
  summary_path     = 'summary.txt'
  description_path = 'description.txt'

  powerjira(
    config_path,
    agent_path,
    summary_path,
    description_path,
    argv[1:])
  exit(0)
'''

# stdlib
from os import path
from pathlib import Path
from typing import List, Dict, Union
from sys import argv, exit, getsizeof
from subprocess import call, check_output
# custom
from toolchain.commands import make_logic, fetch_logic, prune_watched_logic
# 3rd party
try:
  import typer
  from yaml import safe_load, YAMLError
except ModuleNotFoundError as e:
  print("Error: Missing one or more 3rd-party packages (pip install).")
  exit(1)


def powerjira(config_path:str, agent_path:str, summary_path:str, description_path:str, raw:bool=False, fetch_key:str='') -> None:
  app = typer.Typer()

  @app.command()
  def make(
    config_path:str=config_path, agent_path:str=agent_path, summary_path:str=summary_path, description_path:str=description_path
  ) -> None:
    '''Create Ticket

    Create a jira ticket, only `task` and `epic` types are supported.

    ───Params
    config_path:str :: path to config file for build spec
    agent_path:str :: path to creds file
    summary_path:str :: path to text file containing ticket title (jira calls it the summary)
    description_path:str :: path to text file containing ticket description
    '''
    return make_logic(config_path, agent_path, summary_path, description_path)
  
  @app.command()
  def fetch(
    fetch_key:str, agent_path:str=agent_path, raw:bool=False
  ) -> None:
    '''Fetch ticket

    Print a jira ticket, same output as `make` command.

    ───Params
    fetch_key:str :: issue key to search for
    agent_path:str :: path to creds file
    raw:bool :: whether to print raw json dump
    '''
    return fetch_logic(fetch_key, agent_path, raw)
  
  @app.command()
  def prune_watched(
    agent_path:str=agent_path
  ) -> None:
    '''Prune Watched Issues

    Stop watching all watched issues with status DONE.

    ───Params
    agent_path:str :: path to creds file
    '''
    return prune_watched_logic(agent_path)
  
  if (__name__ == "powerjira") or (__name__ == '__main__'):
    app()


## Local Testing
# powerjira(argv)



# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['powerjira', 'commands', 'utils']
install_requires = \
['PyYAML>=6.0,<7.0', 'jira>=3.4.1,<4.0.0', 'typer>=0.9.0,<0.10.0']

setup_kwargs = {
    'name': 'powerjira',
    'version': '0.2.7',
    'description': 'A succinct, minimal local jira control plane that can live in your text editor. Interface with tickets fast!',
    'long_description': '# **PowerJira**\n*A succinct local jira control plane*\n\n<br />\n\n## **Welcome to PowerJira!**\nHate how visually noise and clunky the Jira web-app is? Ever wish you could just type the small subset of issue fields you actually care about into an editor and create/find tickets quickly?\n\nWelcome to the party! 🥳\n\n<br />\n\n### **Table of Contents** 📖\n<hr>\n\n  - [Welcome](#welcome-to-powerjira)\n  - [**Get Started**](#get-started-)\n  - [Usage](#usage-)\n  - [Technologies](#technologies-)\n  - [Contribute](#Contribute-)\n  - [Acknowledgements](#acknowledgements-)\n  - [License/Stats/Author](#license-stats-author-)\n\n<br />\n\n## **Get Started 🚀**\n<hr>\n\nTo get started, clone this repo and populate the config files per the readme.\n\nSpecifically, you need 4 files. These can be named whatever you wish by editing `main.py:59`. \\\nBelow are the requirements of the file with the default (suggested) file names:\n- `summary` - Text file for the issue summary\n- `description` - Text file for the issue description\n- `agent.yml` - Yaml file for user credentials\n- `config.yml` - Yaml file to configure issue creation\n\nThe text files simply contain the text you want in the ticket. \\\nAs for the yaml files, see the **Usage** section below.\n\nSet up your editor space the way you like it, and bam! You\'ve dramatically reduced your time spent in the jira web-app, and even possibly absolve the need for it completely!\n\n<br />\n\n## **Usage ⚙**\n<hr>\n\nWith your shell\'s working directory positioned where the 4 files are present (or if paths have been set accordingly):\n\n**Create ticket with config values**\n```sh\n./main.py [-r|--raw]\n```\n**Fetch info from existing ticket**\n```sh\n./main.py --fetch=<issue-key> [-r|--raw]\n```\n**Info**\n```sh\n./main.py [-h|--help]\n```\n\nThe "*raw*" option flag (`-r`, `--raw`) outputs a formatted dump of the raw api response.\n\nFor your own custom use:\n```python\nfrom powerjira import fetchIssue, createTicket\n```\n\nFor the configuration yaml\'s:\n**config.yml**\n```yaml\nproject:   <project-key>\npriority:  <priority>\n\nepicKey:   # Leave as empty for standalone task or epic\nissueType: <type>\n\nreporter:  <reporter-account-id>\nassignee:  <assignee-account-id>\n```\n**agent.yaml**\n```yaml\ndomain:   <host>\n\nuserName: <email>\ntoken:    <jira-access-token>\n```\n\n<br />\n\n## **Technologies 🧰**\n<hr>\n\n  - [PyYAML](https://pypi.org/project/PyYAML/)\n  - [python-jira](https://pypi.org/project/jira/)\n  - [Poetry](https://python-poetry.org/)\n\n<br />\n\n## **Contribute 🤝**\n<hr>\n\nFeel free to push PR\'s to help make this tool more extensible/flexible.\n\n<br />\n\n## **Acknowledgements 💙**\n<hr>\n\nThanks to Atlassian for making market-leading tools that kinda frustrate me.\n\n<br />\n\n## **License, Stats, Author 📜**\n<hr>\n\n<img align="right" alt="example image tag" src="https://i.imgur.com/jtNwEWu.png" width="200" />\n\n<!-- badge cluster -->\n\n![PyPI](https://img.shields.io/pypi/v/powerjira)\n![GitHub repo size](https://img.shields.io/github/repo-size/anthonybench/powerjira)\n\n<!-- / -->\nSee [License](https://www.gnu.org/licenses/gpl-3.0.txt) for the full license text.\n\nThis repository was authored by *Isaac Yep*. \\\n[PyPi Package](https://pypi.org/project/powerjira/#table-of-contents)\n\n[Back to Table of Contents](#table-of-contents-)',
    'author': 'sleepyboy',
    'author_email': 'anthonybenchyep@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/anthonybench/powerjira',
    'py_modules': modules,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
